@@ -6,17 +6,22 @@ import django
 from django.contrib.contenttypes.models import ContentType
 from django.core import serializers
 from django.core.exceptions import ImproperlyConfigured, ValidationError
+from django.core.management import call_command
+from django.db import connection, models
 from django.test import TestCase, TransactionTestCase
+from django.test.utils import override_settings
+from django.utils import six
 from django.utils.encoding import force_text
 
-from .forms import CustomPKFoodForm, DirectFoodForm, FoodForm, OfficialFoodForm
+from .forms import (CustomPKFoodForm, DirectCustomPKFoodForm, DirectFoodForm,
+                    FoodForm, OfficialFoodForm)
 from .models import (Article, Child, CustomManager, CustomPKFood,
-                     CustomPKHousePet, CustomPKPet, DirectFood,
+                     CustomPKHousePet, CustomPKPet, DirectCustomPKFood,
+                     DirectCustomPKHousePet, DirectCustomPKPet, DirectFood,
                      DirectHousePet, DirectPet, Food, HousePet, Movie,
                      OfficialFood, OfficialHousePet, OfficialPet,
                      OfficialTag, OfficialThroughModel, Pet, Photo,
-                     TaggedCustomPKFood, TaggedCustomPKPet, TaggedFood,
-                     TaggedPet)
+                     TaggedCustomPK, TaggedCustomPKFood, TaggedFood)
 
 from taggit.managers import _model_name, _TaggableManager, TaggableManager
 from taggit.models import Tag, TaggedItem
@@ -100,6 +105,10 @@ class TagModelDirectTestCase(TagModelTestCase):
     food_model = DirectFood
     tag_model = Tag
 
+class TagModelDirectCustomPKTestCase(TagModelTestCase):
+    food_model = DirectCustomPKFood
+    tag_model = Tag
+
 class TagModelCustomPKTestCase(TagModelTestCase):
     food_model = CustomPKFood
     tag_model = Tag
@@ -336,7 +345,14 @@ class TaggableManagerTestCase(BaseTaggingTestCase):
         self.assertTrue(hasattr(field, 'rel'))
         self.assertTrue(hasattr(field.rel, 'to'))
         self.assertTrue(hasattr(field, 'related'))
-        self.assertEqual(self.food_model, field.related.model)
+
+        # This API has changed in Django 1.8
+        # https://code.djangoproject.com/ticket/21414
+        if django.VERSION >= (1, 8):
+            self.assertEqual(self.food_model, field.model)
+            self.assertEqual(self.tag_model, field.related.model)
+        else:
+            self.assertEqual(self.food_model, field.related.model)
 
     def test_names_method(self):
         apple = self.food_model.objects.create(name="apple")
@@ -368,17 +384,49 @@ class TaggableManagerTestCase(BaseTaggingTestCase):
                 'apple': set(['1', '2'])
             })
 
+    def test_internal_type_is_manytomany(self):
+        self.assertEqual(
+            TaggableManager().get_internal_type(), 'ManyToManyField'
+        )
+
+    def test_prefetch_no_extra_join(self):
+        apple = self.food_model.objects.create(name="apple")
+        apple.tags.add('1', '2')
+        with self.assertNumQueries(2):
+            list(self.food_model.objects.prefetch_related('tags').all())
+            join_clause = 'INNER JOIN "%s"' % self.taggeditem_model._meta.db_table
+            self.assertEqual(connection.queries[-1]['sql'].count(join_clause), 1, connection.queries[-2:])
+
+    @override_settings(TAGGIT_CASE_INSENSITIVE=True)
+    def test_with_case_insensitive_option(self):
+        spain = self.tag_model.objects.create(name="Spain", slug="spain")
+        orange = self.food_model.objects.create(name="orange")
+        orange.tags.add('spain')
+        self.assertEqual(list(orange.tags.all()), [spain])
+
+
 class TaggableManagerDirectTestCase(TaggableManagerTestCase):
     food_model = DirectFood
     pet_model = DirectPet
     housepet_model = DirectHousePet
     taggeditem_model = TaggedFood
 
+class TaggableManagerDirectCustomPKTestCase(TaggableManagerTestCase):
+    food_model = DirectCustomPKFood
+    pet_model = DirectCustomPKPet
+    housepet_model = DirectCustomPKHousePet
+    taggeditem_model = TaggedCustomPKFood
+
+    def test_require_pk(self):
+        # TODO with a charfield pk, pk is never None, so taggit has no way to
+        # tell if the instance is saved or not
+        pass
+
 class TaggableManagerCustomPKTestCase(TaggableManagerTestCase):
     food_model = CustomPKFood
     pet_model = CustomPKPet
     housepet_model = CustomPKHousePet
-    taggeditem_model = TaggedCustomPKFood
+    taggeditem_model = TaggedCustomPK
 
     def test_require_pk(self):
         # TODO with a charfield pk, pk is never None, so taggit has no way to
@@ -403,6 +451,15 @@ class TaggableManagerOfficialTestCase(TaggableManagerTestCase):
 
         self.assertEqual(apple, self.food_model.objects.get(tags__official=False))
 
+    def test_get_tags_with_count(self):
+        apple = self.food_model.objects.create(name="apple")
+        apple.tags.add("red", "green", "delicious")
+        pear = self.food_model.objects.create(name="pear")
+        pear.tags.add("green", "delicious")
+
+        tag_info = self.tag_model.objects.filter(officialfood__in=[apple.id, pear.id], name='green').annotate(models.Count('name'))
+        self.assertEqual(tag_info[0].name__count, 2)
+
 class TaggableManagerInitializationTestCase(TaggableManagerTestCase):
     """Make sure manager override defaults and sets correctly."""
     food_model = Food
@@ -468,6 +525,10 @@ class TaggableFormDirectTestCase(TaggableFormTestCase):
     form_class = DirectFoodForm
     food_model = DirectFood
 
+class TaggableFormDirectCustomPKTestCase(TaggableFormTestCase):
+    form_class = DirectCustomPKFoodForm
+    food_model = DirectCustomPKFood
+
 class TaggableFormCustomPKTestCase(TaggableFormTestCase):
     form_class = CustomPKFoodForm
     food_model = CustomPKFood
@@ -561,6 +622,20 @@ class TagStringParseTestCase(UnitTestCase):
         self.assertEqual(edit_string_for_tags([plain, comma]), '"com,ma", plain')
         self.assertEqual(edit_string_for_tags([comma, spaces]), '"com,ma", "spa ces"')
 
+    @override_settings(TAGGIT_TAGS_FROM_STRING='tests.custom_parser.comma_splitter')
+    def test_custom_comma_splitter(self):
+        self.assertEqual(parse_tags('   Cued Speech '), ['Cued Speech'])
+        self.assertEqual(parse_tags(' ,Cued Speech, '), ['Cued Speech'])
+        self.assertEqual(parse_tags('Cued Speech'), ['Cued Speech'])
+        self.assertEqual(parse_tags('Cued Speech, dictionary'),
+                         ['Cued Speech', 'dictionary'])
+
+    @override_settings(TAGGIT_STRING_FROM_TAGS='tests.custom_parser.comma_joiner')
+    def test_custom_comma_joiner(self):
+        a = Tag.objects.create(name='Cued Speech')
+        b = Tag.objects.create(name='transliterator')
+        self.assertEqual(edit_string_for_tags([a, b]), 'Cued Speech, transliterator')
+
 
 @skipIf(django.VERSION < (1, 7), "not relevant for Django < 1.7")
 class DeconstructTestCase(UnitTestCase):
@@ -597,3 +672,12 @@ class InheritedPrefetchTests(TestCase):
         self.assertEquals(4, prefetch_tags.count())
         self.assertEquals(set([t.name for t in no_prefetch_tags]),
                           set([t.name for t in prefetch_tags]))
+
+
+class DjangoCheckTests(UnitTestCase):
+
+    def test_django_checks(self):
+        if django.VERSION >= (1, 6):
+            call_command('check', tag=['models'])
+        else:
+                    call_command('validate')
